<?php

/**
 * @file
 * Callbacks used by hook_commerce_payment_method_info().
 */

/**
 * Payment method callback: settings form.
 */
function commerce_cybersource_sawm_settings_form($settings = array()) {
  $settings = (array) $settings + array(
    'access_key' => '',
    'profile_id' => '',
    'secret_key' => '',
    'transaction_mode' => 'test',
    'transaction_type' => COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_SALE,
    'payment_method' => 'card',
    'locale' => 'en-US',
    'checkout_button' => t('Complete checkout'),
  );

  $form['profile_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Profile ID'),
    '#maxlength' => 7,
    '#required' => TRUE,
    '#description' => t('7-character CyberSource secure acceptance profile ID from the "General Settings" page of your CyberSource Secure Acceptance profile.'),
    '#default_value' => $settings['profile_id'],
  );
  $form['access_key'] = array(
    '#type' => 'password',
    '#title' => t('Access key'),
    '#required' => TRUE,
    '#description' => t('First, shorter, 42-character component of the security key generated in the Secure Acceptance profile section titled "Security Keys".'),
    '#default_value' => $settings['access_key'],
  );
  $form['secret_key'] = array(
    '#type' => 'password',
    '#title' => t('Secret key'),
    '#required' => TRUE,
    '#description' => t('Second, longer, 256-character component of the security key generated in the Secure Acceptance profile section titled "Security Keys".'),
    '#default_value' => $settings['secret_key'],
    '#size' => 256,
    '#maxlength' => 256,
  );
  $form['transaction_mode'] = array(
    '#type' => 'select',
    '#title' => t('Transaction mode'),
    '#description' => t('Transaction mode used for processing orders.'),
    '#options' => array(
      'test' => t('Test'),
      'live' => t('Live'),
    ),
    '#multiple' => FALSE,
    '#default_value' => $settings['transaction_mode'],
  );
  $form['transaction_type'] = array(
    '#type' => 'select',
    '#title' => t('Transaction type'),
    '#options' => array(
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_SALE => t('Capture funds'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_AUTH => t('Authorize funds'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_CREATE_TOKEN => t('Create payment token'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_AUTH_CREATE_TOKEN => t('Authorize funds and create payment token'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_SALE_CREATE_TOKEN => t('Capture funds and create payment token'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_UPDATE_TOKEN => t('Update payment token'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_AUTH_UPDATE_TOKEN => t('Authorize funds and update payment token'),
      COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_SALE_UPDATE_TOKEN => t('Capture funds and update payment token'),
    ),
    '#multiple' => FALSE,
    '#default_value' => $settings['transaction_type'],
  );
  /*
  Secure Acceptance documentation references "27 languages for localization
  purposes", but lists far more codes than that, and only general 24 languages.

  The locale API field is required for submission.
  @code
  <input type="hidden" name="locale" value="en-us">
  @endcode
  @link http://apps.cybersource.com/library/documentation/dev_guides/Secure_Acceptance_WM/Secure_Acceptance_WM.pdf CyberSource Secure Acceptance Web/Mobile September 2013, Page 31, "Localization" @endlink
  */
  $form['locale'] = array(
    '#type' => 'select',
    '#title' => t('Locale'),
    '#options' => array(
      'ar-XN' => t('Arabic'),
      'km-KH' => t('Cambodia'),
      'zh-HK' => t('Chinese - Hong Kong'),
      'zh-MO' => t('Chinese - Maco'),
      'zh-CN' => t('Chinese - Mainland'),
      'zh-SG' => t('Chinese - Singapore'),
      'zh-TW' => t('Chinese - Taiwan'),
      'cz-CZ' => t('Czech'),
      'nl-nl' => t('Dutch'),
      'en-US' => t('English - American'),
      'en-AU' => t('English - Australia'),
      'en-GB' => t('English - Britain'),
      'en-CA' => t('English - Canada'),
      'en-IE' => t('English - Ireland'),
      'en-NZ' => t('English - New Zealand'),
      'fr-FR' => t('French'),
      'fr-CA' => t('French - Canada'),
      'de-DE' => t('German'),
      'de-AT' => t('German - Austria'),
      'hu-HU' => t('Hungary'),
      'id-ID' => t('Indonesian'),
      'it-IT' => t('Italian'),
      'ja-JP' => t('Japanese'),
      'ko-KR' => t('Korean'),
      'lo-LA' => t("Lao People's Democratic Replublic"),
      'ms-MY' => t('Malaysian Bahasa'),
      'tl-PH' => t('Philippines Tagalog'),
      'pl-PL' => t('Polish'),
      'pt-BR' => t('Portuguese - Brazil'),
      'ru-RU' => t('Russian'),
      'sk-SK' => t('Slovakian'),
      'es-ES' => t('Spanish'),
      'es-AR' => t('Spanish - Argentina'),
      'es-CL' => t('Spanish - Chile'),
      'es-CO' => t('Spanish - Colombia'),
      'es-MX' => t('Spanish - Mexico'),
      'es-PE' => t('Spanish - Peru'),
      'es-US' => t('Spanish - American'),
      'th-TH' => t('Thai'),
      'tr-TR' => t('Turkish'),
      'vi-VN' => t('Vietnamese'),
    ),
    '#default_value' => $settings['locale'],
  );
  $form['payment_method'] = array(
    '#type' => 'select',
    '#options' => array('card' => t('Card'), 'echeck' => t('eCheck')),
    '#title' => t('Payment method'),
    '#default_value' => $settings['payment_method'],
  );
  $form['checkout_button'] = array(
    '#type' => 'textfield',
    '#title' => t('Payment submit button text'),
    '#description' => t('Specify the text for the payment form submit button.'),
    '#default_value' => $settings['checkout_button'],
  );
  return $form;
}

/**
 * Payment method callback: submit form submission.
 *
 * Processes payment as necessary using data inputted via the payment details
 * form elements on the form, resulting in the creation of a payment
 * transaction.
 *
 * @param array $payment_method
 *   An array containing CyberSource SAWM info hook values and user settings.
 */
function commerce_cybersource_sawm_submit_form_submit($payment_method, $pane_form, $pane_values, $order, $charge) {
  $order->data['cybersource_sawm'] = $pane_values;

  commerce_cybersource_sawm_transaction($payment_method, $order, $charge);
}

/**
 * Payment method callback.
 *
 * Builds an array of form fields for sending a POST to CyberSource.
 *
 * @return array
 *   Form fields for a POST to CyberSource.
 */
function commerce_cybersource_sawm_redirect_form($form, &$form_state, $order, $payment_method) {
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);

  /*
  @todo
  $cybersource_user_token = md5(microtime() . $order->order_id);
  $order->data['cybersource_sawm_user_token'] = $cybersource_user_token;
  commerce_order_save($order);

  // Add the existing payment token if this is a token update.
  @code
  $user = user_load($order->uid);
  if (strpos($payment_method['settings']['transaction_type'], 'update') !== FALSE && !empty($user->data['commerce_cybersource_sawm_payment_token'])) {
    $form['payment_token'] = array(
      '#type' => 'hidden',
      '#value' => $user->data['commerce_cybersource_sawm_payment_token'],
    );
  }
  @endcode
  */

  /*
  Required fields are listed first except for the signature, which is a hash of
  and can't be set until all the values of other fields are in place.
  */

  // Required. Access key generated in the Business Center SA profile.
  $form['access_key'] = array(
    '#type' => 'hidden',
    '#value' => $payment_method['settings']['access_key'],
  );

  // Required. Currency should be in ISO format.
  $currency_code = $order_wrapper->commerce_order_total->currency_code->value();
  if (empty($currency_code)) {
    $currency_code = commerce_default_currency();
  }
  $form['currency'] = array(
    '#type' => 'hidden',
    '#value' => $currency_code,
  );

  // Required. Add the charge amount if this is an auth or sale transaction.
  $amount = $order_wrapper->commerce_order_total->amount->value();
  if ($payment_method['settings']['transaction_type'] != COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_CREATE_TOKEN && $payment_method['settings']['transaction_type'] != COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_UPDATE_TOKEN) {

    // Allow other modules to alter the transaction amount.
    drupal_alter('commerce_cybersource_sawm_amount', $amount);

    // Should be greater than or equal to zero and include tax.
    $form['amount'] = array(
      '#type' => 'hidden',
      '#value' => number_format(commerce_currency_amount_to_decimal($amount, $currency_code), 2, '.', ''),
    );
  }
  // Required. Language code, with country variant in ISO format, e.g. en-us.
  $form['locale'] = array(
    '#type' => 'hidden',
    '#value' => $payment_method['settings']['locale'],
  );
  // Required. See General Settings in the Secure Acceptance profile.
  $form['profile_id'] = array(
    '#type' => 'hidden',
    '#value' => $payment_method['settings']['profile_id'],
  );
  /*
  Required.
  Get existing transaction id from submitting the payment selection form.

  The uniqid() function is not a UUID, but CyberSource's demo script uses it.
  Since they're not looking for a genuine UUID, we'll send the transaction ID.

  The Commerce CyberSource SASOP module doesn't generate the transaction ID
  until CyberSource makes a request back to the override_custom_receipt_page
  because it uses the uniqid() function here and stores that as metadata on the
  order. By generating a transaction in Commerce before the redirect, the site
  owner can have a log of transactions that may help debug payment issues
  evidenced by having a bunch of pending transactions on an order.
  */
  $transaction_info = db_select('commerce_payment_transaction', 'cpt')
    ->fields('cpt', array('transaction_id', 'remote_id', 'message', 'changed'))
    ->condition('cpt.payment_method', 'cybersource_sawm')
    ->condition('cpt.order_id', $order->order_id)
    ->condition('cpt.status', COMMERCE_PAYMENT_STATUS_PENDING)
    ->orderBy('cpt.created', 'DESC')
    ->range(0, 1)
    ->addTag('commerce_cybersource_sawm_redirect_transaction')
    ->execute()
    ->fetchAssoc();
  if ($transaction_info && is_array($transaction_info)) {
    $transaction = commerce_payment_transaction_load($transaction_info['transaction_id']);
  }
  else {
    watchdog('commerce_cybersource_sawm', 'Failed to load a transaction for order @id.', array('@id' => $order->order_id));
  }
  $form['transaction_uuid'] = array(
    '#type' => 'hidden',
    '#value' => $transaction->transaction_id,
  );

  // Required. CyberSource recommends either an order or transaction ID here.
  $form['reference_number'] = array(
    '#type' => 'hidden',
    '#value' => $order->order_id,
  );

  /*
  Required.
  Date and time signature was generated. Must be in UTC date & time format.
  Used by CyberSource to detect duplicate transaction attempts.
  */
  $form['signed_date_time'] = array(
    '#type' => 'hidden',
    '#value' => gmdate("Y-m-d\TH:i:s\Z"),
  );

  /*
  Required.
  Possible values:
    authorization
    sale
    create_payment_token
    authorization,create_payment_token
    sale,create_payment_token
    authorization,update_payment_token
    sale,update_payment_token
  */
  $form['transaction_type'] = array(
    '#type' => 'hidden',
    '#value' => $payment_method['settings']['transaction_type'],
  );
  // Required. Value is set later. Seeding here for signed field array.
  $form['signed_field_names'] = array(
    '#type' => 'hidden',
  );
  // Required. The ideal case is to have no fields un-signed.
  $form['unsigned_field_names'] = array(
    '#type' => 'hidden',
    // Fields submitted by Drupal, useless to CyberSource.
    '#value' => 'form_build_id,form_id,form_token',
  );
  /*
  Optional.
  The substring is required because the Cybersource API only accepts 15
  characters for IP address; IPv6 addresses are 45 characters in length.
  */
  $form['customer_ip_address'] = array(
    '#type' => 'hidden',
    '#value' => substr(ip_address(), 0, 15),
  );
  // Optional. Possible values: card, echeck. Otherwise use SA profile default.
  $form['payment_method'] = array(
    '#type' => 'hidden',
    '#value' => $payment_method['settings']['payment_method'],
  );
  // Optional. Overrides the custom receipt profile setting in the SA profile.
  $form['override_custom_receipt_page'] = array(
    '#type' => 'hidden',
    '#value' => url('cybersource_sawm/reply', array('absolute' => TRUE)),
  );

  // Prepare the billing address for use in the request.
  $billing_address = $order_wrapper->commerce_customer_billing->commerce_customer_address->value();

  // If there is no first_name field, break apart the name_line field.
  if (empty($billing_address['first_name'])) {
    $name_parts = explode(' ', $billing_address['name_line']);
    $billing_address['first_name'] = array_shift($name_parts);
    $billing_address['last_name'] = implode(' ', $name_parts);
  }

  // Optional. First name.
  $form['bill_to_forename'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['first_name'],
  );
  // Optional. Last name.
  $form['bill_to_surname'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['last_name'],
  );
  // Optional. Company name.
  $form['bill_to_company_name'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['organisation_name'],
  );
  // Optional. Customer email.
  $form['bill_to_email'] = array(
    '#type' => 'hidden',
    '#value' => $order->mail,
  );
  // Optional. Address line 1.
  $form['bill_to_address_line1'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['thoroughfare'],
  );
  // Optional. Address line 2.
  $form['bill_to_address_line2'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['premise'],
  );
  // Optional.
  $form['bill_to_address_city'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['locality'],
  );
  // Optional.
  $form['bill_to_address_state'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['administrative_area'],
  );
  // Optional.
  $form['bill_to_address_country'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['country'],
  );
  // Optional.
  $form['bill_to_address_postal_code'] = array(
    '#type' => 'hidden',
    '#value' => $billing_address['postal_code'],
  );

  /*
  Commerce Kickstart doesn't implement a user profile phone field.
  @code
  $form['bill_to_phone'] = array(
    '#type' => 'hidden',
    '#value' => ?,
  );
  @endcode
  */

  /*
  @todo
  Add a loop here to list items on the request. To list an item's code, this
  module would probably have to hook some metadata into each product throughout
  the Commerce store or otherwise list "default" for all items.
  @code
  // CyberSource limits the items listed on the invoice to 0-49.
  for ($i=0, $j=count($order->commerce_line_items); $i<$j && $i<50; $i++) {
    $form["item_$i_name"] = array(
      '#type' => 'hidden',
      '#value' => ,
    );
    $form["item_$i_quantity"] = array(
      '#type' => 'hidden',
      '#value' => ,
    );
    $form["item_$i_sku"] = array(
      '#type' => 'hidden',
      '#value' => ,
    );
    $form["item_$i_unit_price"] = array(
      '#type' => 'hidden',
      '#value' => ,
    );
    // One of the following options:
    // default, adult_content, coupon, electronic_good, electronic_software,
    // gift_certificate, service, subscription, handling_only, service
    // shipping_and_handling, shipping_only
    $form["item_$i_code"] = array(
      '#type' => 'hidden',
      '#value' => ,
    );
  }
  @endcode
  */

  /*
  All fields should be added to $form by now so they can be signed.
  The signed_field_names is a required field and should list itself.
  */
  $signed_field_names_list = implode(',', array_keys($form));
  $form['signed_field_names']['#value'] = $signed_field_names_list;

  /*
  Add the signature field of signed name value pairs.
  Generated as SHA256 base64 using the secret key.
  */
  $form['signature'] = array(
    '#type' => 'hidden',
    '#value' => commerce_cybersource_sawm_sign($form, $payment_method['settings']['secret_key']),
  );

  // Set appropriate server path after other fields get signed.
  if ($payment_method['settings']['transaction_type'] == COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_CREATE_TOKEN ||
    $payment_method['settings']['transaction_type'] == COMMERCE_CYBERSOURCE_SAWM_TRANSACTION_UPDATE_TOKEN) {
    $testing_url = COMMERCE_CYBERSOURCE_SAWM_TEST_TOKEN_SERVER;
    $live_url = COMMERCE_CYBERSOURCE_SAWM_LIVE_TOKEN_SERVER;
  }
  else {
    $testing_url = COMMERCE_CYBERSOURCE_SAWM_TEST_TRANSACTION_SERVER;
    $live_url = COMMERCE_CYBERSOURCE_SAWM_LIVE_TRANSACTION_SERVER;
  }

  // Set the form action to the appropriate CyberSource URL.
  switch ($payment_method['settings']['transaction_mode']) {
    case 'live':
      $url = $live_url;
      break;

    case 'test':
    default:
      $url = $testing_url;
  }

  $form['#action'] = $url;

  // Allow other modules an opportunity to add to or alter the transaction data.
  drupal_alter('commerce_cybersource_sawm_transaction_data', $form);

  return $form;
}
